# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
systeminfo:
    python interface to functions in libsmbios_c  system_info.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNullPtrFN, errorOnNegativeFN, errorOnZeroFN, c_utf8_p
from .trace_decorator import traceLog, getLog

__all__ = ["DellSmi", "DELL_SMI_DEFAULTS", "DELL_SMI_GET_SINGLETON", "DELL_SMI_GET_NEW", "DELL_SMI_UNIT_TEST_MODE"]
__all__.extend( [ "cbARG1", "cbARG2", "cbARG3", "cbARG4", "cbRES1", "cbRES2", "cbRES3", "cbRES4", ])

cbARG1=0
cbARG2=1
cbARG3=2
cbARG4=3
cbRES1=0
cbRES2=1
cbRES3=2
cbRES4=3
DELL_SMI_DEFAULTS      =0x0000
DELL_SMI_GET_SINGLETON =0x0001
DELL_SMI_GET_NEW       =0x0002
DELL_SMI_UNIT_TEST_MODE=0x0004

class SMIExecutionError(Exception): pass # ret = -1
class SMIUnsupported(Exception): pass # ret = -2
class SMIUnhandled(Exception): pass   # ret = -3
class SMIOutputBufferFormatError(Exception): pass  # ret == -5
class SMIOutputBufferTooSmall(Exception): pass  # ret == -6

class BadPassword(Exception): pass
class SmiCreateError(Exception): pass
class SmiBufferCreateError(Exception): pass


@traceLog()
def DellSmi(flags=DELL_SMI_GET_NEW, *args):
    if _DellSmi._instance is None:
        _DellSmi._instance = _DellSmi( flags, *args)
    return _DellSmi._instance

class _DellSmi(ctypes.Structure):
    _instance = None

    @traceLog()
    def __init__(self, *args, **kargs):
        self._smiobj = None
        self._smiobj = DLL.dell_smi_factory(*args)
        self.bufs = [0,0,0,0]

    # dont decorate __del__
    def __del__(self):
        DLL.dell_smi_obj_free(self._smiobj)

    @traceLog()
    def setClass(self, smiclass):
        DLL.dell_smi_obj_set_class(self._smiobj, smiclass)

    @traceLog()
    def setSelect(self, select):
        DLL.dell_smi_obj_set_select(self._smiobj, select)

    @traceLog()
    def setArg(self, arg, val):
        DLL.dell_smi_obj_set_arg(self._smiobj, arg, val)

    @traceLog()
    def getRes(self, res):
        return DLL.dell_smi_obj_get_res(self._smiobj, res)

    @traceLog()
    def buffer_frombios_auto(self, arg, size):
        self.bufs[arg] = DLL.dell_smi_obj_make_buffer_frombios_auto(self._smiobj, arg, size)
        return self.bufs[arg]

    @traceLog()
    def buffer_frombios_withheader(self, arg, size):
        self.bufs[arg] = DLL.dell_smi_obj_make_buffer_frombios_withheader(self._smiobj, arg, size)
        return self.bufs[arg]

    @traceLog()
    def buffer_frombios_withoutheader(self, arg, size):
        self.bufs[arg] = DLL.dell_smi_obj_make_buffer_frombios_withoutheader(self._smiobj, arg, size)
        return self.bufs[arg]

    @traceLog()
    def buffer_tobios(self, arg, size, buf):
        self.bufs[arg] = DLL.dell_smi_obj_make_buffer_tobios(self._smiobj, arg, size)
        if len(buf) < size: size = len(buf)
        ctypes.memmove( self.bufs[arg], buf, size )

    @traceLog()
    def execute(self):
        ret = DLL.dell_smi_obj_execute(self._smiobj)
        raiseExceptionOnError(ret, self)

    @traceLog()
    def getBufContents(self, arg):
        return self.bufs[arg]

@traceLog()
def raiseExceptionOnError(ret, smiobj=None):
    if ret == -1:
        raise SMIExecutionError( _("SMI executed but returned error"))
    if ret == -2:
        raise SMIUnsupported(_("SMI is unsupported"))
    if ret == -3:
        raise SMIUnhandled(_("SMI did not execute"))
    if ret == -5:
        raise SMIOutputBufferFormatError(_("The output buffer the BIOS requires is in the wrong format"))
    if ret == -6:
        raise SMIOutputBufferTooSmall(_("The output buffer the BIOS requires is too small"))


# define type that can be used for arg/res:  u32 arg[4]
array_4_u32 = ctypes.c_int32 * 4

# define strerror first so we can use it in error checking other functions.
DLL.dell_smi_strerror.argtypes = [ ]
DLL.dell_smi_strerror.restype = c_utf8_p
@traceLog()
def _strerror():
    return DLL.dell_smi_strerror()

#void dell_simple_ci_smi(u16 smiClass, u16 select, const u32 args[4], u32 res[4]);
DLL.dell_simple_ci_smi.argtypes = [ctypes.c_uint16, ctypes.c_uint16, array_4_u32, array_4_u32]
DLL.dell_simple_ci_smi.restype = ctypes.c_int
DLL.dell_simple_ci_smi.errcheck = errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
@traceLog()
def simple_ci_smi(smiClass, select, *args):
    arg = array_4_u32(*args)
    res = array_4_u32(0, 0, 0, 0)
    DLL.dell_simple_ci_smi(smiClass, select, arg, res)
    return [ i for i in res ]
__all__.append("simple_ci_smi")

#int dell_smi_read_nv_storage         (u32 location, u32 *minValue, u32 *maxValue);
DLL.dell_smi_read_nv_storage.errcheck = errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
DLL.dell_smi_read_nv_storage.restype = ctypes.c_int
DLL.dell_smi_read_nv_storage.argtypes = [
        ctypes.c_uint32,
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32)]
@traceLog()
def read_nv_storage(location):
    min = ctypes.c_uint32(0)
    max = ctypes.c_uint32(0)
    cur = ctypes.c_uint32(0)
    DLL.dell_smi_read_nv_storage(location, ctypes.byref(cur), ctypes.byref(min), ctypes.byref(max))
    return (cur.value, min.value, max.value)
__all__.append("read_nv_storage")

#int dell_smi_read_battery_mode_setting(u32 location, u32 *minValue, u32 *maxValue);
DLL.dell_smi_read_battery_mode_setting.errcheck = errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
DLL.dell_smi_read_battery_mode_setting.restype = ctypes.c_int
DLL.dell_smi_read_battery_mode_setting.argtypes = [
        ctypes.c_uint32,
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32)]
@traceLog()
def read_battery_mode_setting(location):
    min = ctypes.c_uint32(0)
    max = ctypes.c_uint32(0)
    cur = ctypes.c_uint32(0)
    DLL.dell_smi_read_battery_mode_setting(location, ctypes.byref(cur), ctypes.byref(min), ctypes.byref(max))
    return (cur.value, min.value, max.value)
__all__.append("read_battery_mode_setting")

#int dell_smi_read_ac_mode_setting     (u32 location, u32 *minValue, u32 *maxValue);
DLL.dell_smi_read_ac_mode_setting.errcheck = errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
DLL.dell_smi_read_ac_mode_setting.restype = ctypes.c_int
DLL.dell_smi_read_ac_mode_setting.argtypes = [
        ctypes.c_uint32,
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32),
        ctypes.POINTER(ctypes.c_uint32)]
@traceLog()
def read_ac_mode_setting(location):
    min = ctypes.c_uint32(0)
    max = ctypes.c_uint32(0)
    cur = ctypes.c_uint32(0)
    DLL.dell_smi_read_ac_mode_setting(location, ctypes.byref(cur), ctypes.byref(min), ctypes.byref(max))
    return (cur.value, min.value, max.value)
__all__.append("read_ac_mode_setting")


#int dell_smi_write_nv_storage         (u16 security_key, u32 location, u32 value, u32 *smiret);
DLL.dell_smi_write_nv_storage.argtypes = [ctypes.c_uint16, ctypes.c_uint32, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint32)]
DLL.dell_smi_write_nv_storage.restype = ctypes.c_int
DLL.dell_smi_write_nv_storage.errcheck=errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
write_nv_storage = DLL.dell_smi_write_nv_storage
__all__.append("write_nv_storage")

#int dell_smi_write_battery_mode_setting(u16 security_key, u32 location, u32 value, u32 *smiret);
DLL.dell_smi_write_battery_mode_setting.argtypes = [ctypes.c_uint16, ctypes.c_uint32, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint32)]
DLL.dell_smi_write_battery_mode_setting.restype = ctypes.c_int
DLL.dell_smi_write_battery_mode_setting.errcheck=errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
write_battery_mode_setting = DLL.dell_smi_write_battery_mode_setting
__all__.append("write_battery_mode_setting")

#int dell_smi_write_ac_mode_setting     (u16 security_key, u32 location, u32 value, u32 *smiret);
DLL.dell_smi_write_ac_mode_setting.argtypes = [ctypes.c_uint16, ctypes.c_uint32, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint32)]
DLL.dell_smi_write_ac_mode_setting.restype = ctypes.c_int
DLL.dell_smi_write_ac_mode_setting.errcheck=errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_strerror()))
write_ac_mode_setting = DLL.dell_smi_write_ac_mode_setting
__all__.append("write_ac_mode_setting")

@traceLog()
def securityException(r):
    if r == -1:
        return BadPassword()
    if r == -2:
        return SMIExecutionError()

#int dell_smi_get_security_key(const char *pass_scancode, u16 *security_key);
DLL.dell_smi_get_security_key.argtypes = [ctypes.c_char_p, ctypes.POINTER(ctypes.c_uint16)]
DLL.dell_smi_get_security_key.restype = ctypes.c_int
DLL.dell_smi_get_security_key.errcheck=errorOnNegativeFN(lambda r,f,a: securityException(r))
@traceLog()
def get_security_key(password):
    key = ctypes.c_uint16(0)
    DLL.dell_smi_get_security_key(password, key)
    return key.value
__all__.append("get_security_key")


DELL_SMI_PASSWORD_USER = 9
DELL_SMI_PASSWORD_ADMIN = 10
DELL_SMI_PASSWORD_OWNER = 12
DELL_SMI_PASSWORD_FMT_SCANCODE = 0
DELL_SMI_PASSWORD_FMT_ASCII = 1

#int dell_smi_password_format(int which);
DLL.dell_smi_password_format.argtypes = [ctypes.c_int]
DLL.dell_smi_password_format.restype = ctypes.c_int
password_format = DLL.dell_smi_password_format
__all__.append("password_format")

#bool dell_smi_is_password_present(int which);
DLL.dell_smi_is_password_present.argtypes = [ctypes.c_int]
DLL.dell_smi_is_password_present.restype = ctypes.c_bool
is_password_present = DLL.dell_smi_is_password_present
__all__.append("is_password_present")

#int dell_smi_password_verify(int which, const char *password);
# returns:  0==failed, 1==correct password, 2==no password installed
DLL.dell_smi_password_verify.argtypes = [ctypes.c_int, ctypes.c_char_p]
DLL.dell_smi_password_verify.restype = ctypes.c_int
password_verify = DLL.dell_smi_password_verify
__all__.append("password_verify")

#int dell_smi_password_max_len(int which);
DLL.dell_smi_password_max_len.argtypes = [ctypes.c_int]
DLL.dell_smi_password_max_len.restype = ctypes.c_int
password_max_len = DLL.dell_smi_password_max_len
__all__.append("password_max_len")

#int dell_smi_password_min_len(int which);
DLL.dell_smi_password_min_len.argtypes = [ctypes.c_int]
DLL.dell_smi_password_min_len.restype = ctypes.c_int
password_min_len = DLL.dell_smi_password_min_len
__all__.append("password_min_len")

#int dell_smi_password_change(int which, const char *oldpass, const char *newpass);
DLL.dell_smi_password_change.argtypes = [ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p]
DLL.dell_smi_password_change.restype = ctypes.c_int
password_change = DLL.dell_smi_password_change
__all__.append("password_change")


################################################################################
################################################################################
###                                                                          ###
###        SMI Object functions, not exported                                ###
###                                                                          ###
################################################################################
################################################################################

#struct dell_smi_obj;

# define strerror first so we can use it in error checking other functions.
DLL.dell_smi_obj_strerror.argtypes = [ ctypes.POINTER(_DellSmi) ]
DLL.dell_smi_obj_strerror.restype = c_utf8_p
@traceLog()
def _obj_strerror(obj):
    return DLL.dell_smi_obj_strerror(obj)

#struct dell_smi_obj *dell_smi_factory(int flags, ...);
# dont define argtypes because this is a varargs function...
#DLL.dell_smi_factory.argtypes = [ctypes.c_int, ]
DLL.dell_smi_factory.restype = ctypes.POINTER(_DellSmi)
DLL.dell_smi_factory.errcheck = errorOnNullPtrFN(lambda r,f,a: SmiCreateError(_obj_strerror(r)))

#void dell_smi_obj_free(struct dell_smi_obj *);
DLL.dell_smi_obj_free.argtypes = [ ctypes.POINTER(_DellSmi) ]
DLL.dell_smi_obj_free.restype = None

#void dell_smi_obj_set_class(struct dell_smi_obj *, u16 );
DLL.dell_smi_obj_set_class.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint16 ]
DLL.dell_smi_obj_set_class.restype = None

#void dell_smi_obj_set_select(struct dell_smi_obj *, u16 );
DLL.dell_smi_obj_set_select.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint16 ]
DLL.dell_smi_obj_set_select.restype = None

#void dell_smi_obj_set_arg(struct dell_smi_obj *, u8 argno, u32 value);
DLL.dell_smi_obj_set_arg.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8, ctypes.c_uint32 ]
DLL.dell_smi_obj_set_arg.restype = None

#u32  dell_smi_obj_get_res(struct dell_smi_obj *, u8 argno);
DLL.dell_smi_obj_get_res.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8 ]
DLL.dell_smi_obj_get_res.restype = ctypes.c_uint32

#u8  *dell_smi_obj_make_buffer_frombios_auto(struct dell_smi_obj *, u8 argno, size_t size);
DLL.dell_smi_obj_make_buffer_frombios_auto.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8, ctypes.c_size_t ]
DLL.dell_smi_obj_make_buffer_frombios_auto.restype = ctypes.c_void_p
DLL.dell_smi_obj_make_buffer_frombios_auto.errcheck = errorOnNullPtrFN(lambda r,f,a: SmiBufferCreateError(_obj_strerror(a[0])))

#u8  *dell_smi_obj_make_buffer_frombios_withheader(struct dell_smi_obj *, u8 argno, size_t size);
DLL.dell_smi_obj_make_buffer_frombios_withheader.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8, ctypes.c_size_t ]
DLL.dell_smi_obj_make_buffer_frombios_withheader.restype = ctypes.c_void_p
DLL.dell_smi_obj_make_buffer_frombios_withheader.errcheck = errorOnNullPtrFN(lambda r,f,a: SmiBufferCreateError(_obj_strerror(a[0])))

#u8  *dell_smi_obj_make_buffer_frombios_withoutheader(struct dell_smi_obj *, u8 argno, size_t size);
DLL.dell_smi_obj_make_buffer_frombios_withoutheader.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8, ctypes.c_size_t ]
DLL.dell_smi_obj_make_buffer_frombios_withoutheader.restype = ctypes.c_void_p
DLL.dell_smi_obj_make_buffer_frombios_withoutheader.errcheck = errorOnNullPtrFN(lambda r,f,a: SmiBufferCreateError(_obj_strerror(a[0])))

#u8  *dell_smi_obj_make_buffer_tobios(struct dell_smi_obj *, u8 argno, size_t size);
DLL.dell_smi_obj_make_buffer_tobios.argtypes = [ ctypes.POINTER(_DellSmi), ctypes.c_uint8, ctypes.c_size_t ]
DLL.dell_smi_obj_make_buffer_tobios.restype = ctypes.c_void_p
DLL.dell_smi_obj_make_buffer_tobios.errcheck = errorOnNullPtrFN(lambda r,f,a: SmiBufferCreateError(_obj_strerror(a[0])))

#void dell_smi_obj_execute(struct dell_smi_obj *);
DLL.dell_smi_obj_execute.argtypes = [ ctypes.POINTER(_DellSmi) ]
DLL.dell_smi_obj_execute.restype = ctypes.c_int
# dont use error check since class does this
#DLL.dell_smi_obj_execute.errcheck = errorOnNegativeFN(lambda r,f,a: SMIExecutionError(_obj_strerror(a[0])))



# for testing only. It only does en_US, which is just wrong.
@traceLog()
def asc_to_scancode(s):
    return "".join([ chr(asc_to_scancode_map[ord(i)]) for i in s ])

asc_to_scancode_map = [
        0x03, 0x1E, 0x30, 0x46,  0x20, 0x12, 0x21, 0x22,
        0x0E, 0x0F, 0x1C, 0x25,  0x26, 0x1C, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x07, 0x0C,
        0x39, 0x02, 0x28, 0x04,  0x05, 0x06, 0x08, 0x28,
        0x0A, 0x0B, 0x09, 0x0D,  0x33, 0x0C, 0x34, 0x35,
        0x0B, 0x02, 0x03, 0x04,  0x05, 0x06, 0x07, 0x08,
        0x09, 0x0A, 0x27, 0x27,  0x33, 0x0D, 0x34, 0x35,
        0x03, 0x1E, 0x30, 0x2E,  0x20, 0x12, 0x21, 0x22,
        0x23, 0x17, 0x24, 0x25,  0x26, 0x32, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x07, 0x0C,
        0x29, 0x1E, 0x30, 0x2E,  0x20, 0x12, 0x21, 0x22,
        0x23, 0x17, 0x24, 0x25,  0x26, 0x32, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x29, 0x0E,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00
    ]



import logging
import logging.config
from optparse import OptionParser
import os
import sys

from libsmbios_c import memory, smi, cmos, pkgconfdir, localedir, GETTEXT_PACKAGE
from libsmbios_c.trace_decorator import makePrintable

import gettext
gettext.install(GETTEXT_PACKAGE, localedir)

standardFailMessage = _("\n\
Common problems are:\n\
\n\
    -- 'SMM Mitigations' is enabled in BIOS setup.\n\
        Run kernel 4.15 or later with \n\
        dell-smbios-wmi enabled.\n\
         or \n\
        Disable 'SMM mitigations' in BIOS setup.\n\
\n\
    -- Insufficient permissions to perform operation.\n\
       Try running as a more privileged account.\n\
          Linux  : run as 'root' user\n\
          Windows: run as 'administrator' user\n\
\n\
    -- dell-smbios-wmi driver not loaded \n\
       Try loading the dell-smbios-wmi driver \n\
          Linux : modprobe dell-smbios-wmi \n\
\n\
    -- Call filtered by Linux kernel \n\
       Some functionality is natively supported \n\
       by the Linux kernel \n\
\n\
    -- dcdbas device driver not loaded.\n\
       Try loading the dcdbas driver\n\
          Linux  : modprobe dcdbas\n\
")

def getStdOptionParser(usage, version):
    parser = OptionParser(usage=usage, version=version)
    return addStdOptions(parser)

def addStdOptions(parser, passwordOpts=True, securityKeyOpt=False):
    parser.add_option("-v", "--verbose", action="count", dest="verbosity", default=1, help=_("Display more verbose output."))
    parser.add_option("-q", "--quiet", action="store_const", const=0, dest="verbosity", help=_("Minimize program output. Only errors and warnings are displayed."))
    parser.add_option("--trace", action="store_true", dest="trace", default=False, help=_("Enable verbose function tracing."))
    parser.add_option("--logconfig", action="store", default=os.path.join(pkgconfdir, "logging.conf"), help=_("Specify alternate config log."))

    if securityKeyOpt:
        parser.add_option('--security-key', action="store", dest="security_key", default=None, help=_("BIOS pre-calculated security key."))

    if passwordOpts:
        parser.add_option('--password', action="store", dest="password",
                      help=_("BIOS Setup password for set/activate operations."))
        parser.add_option('-r', '--rawpassword', action="store_true", dest="raw", default=False,
                      help=_("Do not auto-convert password to scancodes."))

    parser.add_option('--memory-dat', action="store", type="string", dest="memory_dat",
                      help=_("Path to a memory dump file to use instead of real RAM"),
                      default=None)
    parser.add_option('--cmos-dat', action="store", type="string", dest="cmos_dat",
                      help=_("Path to a CMOS dump file to use instead of real CMOS"),
                      default=None)
    return parser

def setup_std_options(options):
    if options.memory_dat is not None:
        memory.MemoryAccess( memory.MEMORY_GET_SINGLETON | memory.MEMORY_UNIT_TEST_MODE, options.memory_dat.encode('utf-8'))

    if options.cmos_dat is not None:
        cmos.CmosAccess( cmos.CMOS_GET_SINGLETON | cmos.CMOS_UNIT_TEST_MODE, options.cmos_dat.encode('utf-8'))

    options.password_scancode = None
    if options.password is not None:
        options.password_scancode = braindead_asc_to_scancode(options.password)
    options.password_ascii = options.password
    if options.password_ascii:
        options.password_ascii = options.password_ascii.encode('utf-8')
    if options.raw:
        options.password_scancode = options.password
    if options.password_scancode:
        options.password_scancode = options.password_scancode.encode('utf-8')

    if getattr(options, "security_key", None) is not None:
        options.security_key = int(options.security_key, 0)


    setupLogging(options.logconfig, options.verbosity, options.trace)


def setupLogging(configFile, verbosity=1, trace=0):
    # set up logging
    logging.basicConfig()
    logging.raiseExceptions = 0
    if configFile:
        logging.config.fileConfig(configFile)

    root_log    = logging.getLogger()
    log         = logging.getLogger("libsmbios_c")
    verbose_log = logging.getLogger("verbose")
    trace_log   = logging.getLogger("trace")

    log.propagate = 0
    trace_log.propagate = 0
    verbose_log.propagate = 0

    if verbosity >= 1:
        log.propagate = 1
    if verbosity >= 2:
        verbose_log.propagate = 1
    if verbosity >= 3:
        for hdlr in root_log.handlers:
            hdlr.setLevel(logging.DEBUG)
    if trace:
        trace_log.propagate = 1


def wrap(s, line_len=80, indent=0, first_line_indent=0, first_line_start=0):
    sys.stdout.write(" "*first_line_indent)
    chars_printed = first_line_start
    for c in s:
        sys.stdout.write(c)
        chars_printed = chars_printed + 1
        if chars_printed >= line_len:
            sys.stdout.write("\n")
            chars_printed=indent
            sys.stdout.write(" "*indent)

def getSecurityKey(options):
    if options.security_key is None:
        fmt = smi.password_format(smi.DELL_SMI_PASSWORD_ADMIN)
        passToTry = options.password_ascii
        if fmt == smi.DELL_SMI_PASSWORD_FMT_SCANCODE:
            passToTry = options.password_scancode

        options.security_key = smi.get_security_key( passToTry )
    return options.security_key

# for testing only. It only does en_US, which is just wrong.
def braindead_asc_to_scancode(s):
    return "".join([ chr(asc_to_scancode_map[ord(i)]) for i in s ])

asc_to_scancode_map = [
        0x03, 0x1E, 0x30, 0x46,  0x20, 0x12, 0x21, 0x22,
        0x0E, 0x0F, 0x1C, 0x25,  0x26, 0x1C, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x07, 0x0C,
        0x39, 0x02, 0x28, 0x04,  0x05, 0x06, 0x08, 0x28,
        0x0A, 0x0B, 0x09, 0x0D,  0x33, 0x0C, 0x34, 0x35,
        0x0B, 0x02, 0x03, 0x04,  0x05, 0x06, 0x07, 0x08,
        0x09, 0x0A, 0x27, 0x27,  0x33, 0x0D, 0x34, 0x35,
        0x03, 0x1E, 0x30, 0x2E,  0x20, 0x12, 0x21, 0x22,
        0x23, 0x17, 0x24, 0x25,  0x26, 0x32, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x07, 0x0C,
        0x29, 0x1E, 0x30, 0x2E,  0x20, 0x12, 0x21, 0x22,
        0x23, 0x17, 0x24, 0x25,  0x26, 0x32, 0x31, 0x18,
        0x19, 0x10, 0x13, 0x1F,  0x14, 0x16, 0x2F, 0x11,
        0x2D, 0x15, 0x2C, 0x1A,  0x2B, 0x1B, 0x29, 0x0E,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00
    ]



# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
token:
    python interface to functions in libsmbios_c  obj/token.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNullPtrFN, errorOnNegativeFN, freeLibStringFN, c_utf8_p
from .trace_decorator import traceLog, getLog

__all__ = ["TokenTable", "TOKEN_DEFAULTS", "TOKEN_GET_SINGLETON", "TOKEN_GET_NEW", "TOKEN_UNIT_TEST_MODE"]

TOKEN_DEFAULTS      =0x0000
TOKEN_GET_SINGLETON =0x0001
TOKEN_GET_NEW       =0x0002
TOKEN_UNIT_TEST_MODE=0x0004

class TokenTableParseError(Exception): pass
class TokenManipulationFailure(Exception): pass

class Token(ctypes.Structure):
    @traceLog()
    def getId(self):
        return DLL.token_obj_get_id(self)

    # dont trace or we recurse...
    def __repr__(self):
        return "<libsmbios_c.Token ID 0x%04x>" % DLL.token_obj_get_id(self)

    @traceLog()
    def getType(self):
        return DLL.token_obj_get_type(self)

    @traceLog()
    def isBool(self):
        return DLL.token_obj_is_bool(self)

    @traceLog()
    def isActive(self):
        return DLL.token_obj_is_active(self)

    @traceLog()
    def activate(self):
        return DLL.token_obj_activate(self)

    @traceLog()
    def isString(self):
        return DLL.token_obj_is_string(self)

    @traceLog()
    def getPtr(self):
        ptr = DLL.token_obj_get_ptr(self)
        typ = ctypes.POINTER(TokenPtr.subclasses[ self.getType() ])
        return ctypes.cast(ptr, typ).contents

    @traceLog()
    def getString(self):
        len = ctypes.c_size_t()
        retstr = DLL.token_obj_get_string(self, ctypes.byref(len))
        # not usually null-terminated, so use string_at with len
        if bool(retstr): # avoid null-ptr deref
            return ctypes.string_at( retstr, len.value )
        else:
            return None

    @traceLog()
    def setString(self, newstr):
        return DLL.token_obj_set_string(self, newstr, len(newstr))

    @traceLog()
    def tryPassword(self, pass_ascii, pass_scancode):
        return DLL.token_obj_try_password(self, pass_ascii, pass_scancode)


def TokenPtrSubClass(kind):
    def decorator(cls):
        TokenPtr.subclasses[kind] = cls
        return cls
    return decorator

class TokenPtr(ctypes.Structure):
    subclasses = {}
    _pack_ = 1
    _fields_ = []
    def __repr__(self): print("<token ptr>")
    def __str__(self): print("<token ptr>")

@TokenPtrSubClass(0xD4)
class _TokenD4(ctypes.Structure):
    _pack_ = 1
    _fields_ = [ ("tokenId", ctypes.c_uint16), ("location", ctypes.c_uint16), ("value", ctypes.c_uint16)]

@TokenPtrSubClass(0xDA)
class _TokenDA(ctypes.Structure):
    _pack_ = 1
    _fields_ = [ ("tokenId", ctypes.c_uint16), ("location", ctypes.c_uint8), ("andMask", ctypes.c_uint8), ("orValue", ctypes.c_uint8)]

@traceLog()
def TokenTable(flags=TOKEN_GET_SINGLETON, factory_args=None):
    if factory_args is None: factory_args = []
    if _TokenTable._instance is None:
        _TokenTable._instance = _TokenTable( flags, *factory_args)
    return _TokenTable._instance

class _TokenTable(ctypes.Structure):
    _instance = None
    @traceLog()
    def __init__(self, *args):
        self._tableobj = None
        self._tableobj = DLL.token_table_factory(*args)

    def __del__(self):
        if self._tableobj is not None:
            DLL.token_table_free(self._tableobj)

    @traceLog()
    def __iter__(self):
        cur = ctypes.POINTER(Token)()
        while 1:
            cur =DLL.token_table_get_next( self._tableobj, cur )
            if bool(cur):
                yield cur.contents
            else:
                raise StopIteration

    @traceLog()
    def __getitem__(self, id):
        if id is None:
            raise IndexError(_("Cannot dereference NULL ID") )
        cur = ctypes.POINTER(Token)()
        cur =DLL.token_table_get_next_by_id( self._tableobj, cur, id )
        if bool(cur):
            return cur.contents
        else:
            raise IndexError(_("SMBIOS Token ID 0x%04x not found") % id )



#// format error string
#const char *token_table_strerror(const struct token_table *m);
# define strerror first so we can use it in error checking other functions.
DLL.token_table_strerror.argtypes = [ ctypes.POINTER(_TokenTable) ]
DLL.token_table_strerror.restype = c_utf8_p
@traceLog()
def _table_strerror(obj):
    return DLL.token_table_strerror(obj)

#const char *token_obj_strerror(const struct token_table *m);
# define strerror first so we can use it in error checking other functions.
DLL.token_obj_strerror.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_strerror.restype = c_utf8_p
@traceLog()
def _obj_strerror(obj):
    return DLL.token_obj_strerror(obj)

#struct token_table *token_table_factory(int flags, ...);
# dont define argtypes because this is a varargs function...
#DLL.token_table_factory.argtypes = [ctypes.c_int, ]
DLL.token_table_factory.restype = ctypes.POINTER(_TokenTable)
DLL.token_table_factory.errcheck = errorOnNullPtrFN(lambda r,f,a: TokenTableParseError(_table_strerror(r)))

#void   token_table_free(struct token_table *);
DLL.token_table_free.argtypes = [ ctypes.POINTER(_TokenTable) ]
DLL.token_table_free.restype = None

#const struct token_obj * DLL_SPEC token_table_get_next(const struct token_table *, const struct token_obj *cur);
DLL.token_table_get_next.argtypes = [ ctypes.POINTER(_TokenTable), ctypes.POINTER(Token) ]
DLL.token_table_get_next.restype = ctypes.POINTER(Token)

#const struct token_obj * DLL_SPEC token_table_get_next_by_id(const struct token_table *, const struct token_obj *cur, u16 id);
DLL.token_table_get_next_by_id.argtypes = [ ctypes.POINTER(_TokenTable), ctypes.POINTER(Token), ctypes.c_uint16 ]
DLL.token_table_get_next_by_id.restype = ctypes.POINTER(Token)

#u16  DLL_SPEC token_obj_get_id(const struct token_obj *);
DLL.token_obj_get_id.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_get_id.restype = ctypes.c_uint16

#int  DLL_SPEC token_obj_get_type(const struct token_obj *);
DLL.token_obj_get_type.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_get_type.restype = ctypes.c_int

#bool  DLL_SPEC token_obj_is_bool(const struct token_obj *);
DLL.token_obj_is_bool.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_is_bool.restype = ctypes.c_bool

#bool  DLL_SPEC token_obj_is_active(const struct token_obj *);
DLL.token_obj_is_active.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_is_active.restype = ctypes.c_int
DLL.token_obj_is_active.errcheck = errorOnNegativeFN(lambda r,f,a: TokenManipulationFailure(_obj_strerror(a[0])))

#int  DLL_SPEC token_obj_activate(const struct token_obj *);
DLL.token_obj_activate.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_activate.restype = ctypes.c_int
DLL.token_obj_activate.errcheck = errorOnNegativeFN(lambda r,f,a: TokenManipulationFailure(_obj_strerror(a[0])))

#bool  DLL_SPEC token_obj_is_string(const struct token_obj *);
DLL.token_obj_is_string.argtypes = [ ctypes.POINTER(Token) ]
DLL.token_obj_is_string.restype = ctypes.c_bool

@traceLog()
def customFree(result, func, args):
    getLog(prefix="trace.").info("RAN CTYPES FUNCTION: %s" % func.__name__)
    size = args[1]._obj.value
    pystr = ""
    if bool(result) and size >= 0:
        pystr = result[0:size]
        DLL.token_string_free(result)
    else:
        raise TokenManipulationFailure(_obj_strerror(a[0]))
    return pystr

#char*  DLL_SPEC token_obj_get_string(const struct token_obj *, size_t *len);
DLL.token_obj_get_string.argtypes = [ ctypes.POINTER(Token), ctypes.POINTER(ctypes.c_size_t) ]
DLL.token_obj_get_string.restype = ctypes.POINTER(ctypes.c_char)
DLL.token_obj_get_string.errcheck=customFree

DLL.token_string_free.argtypes = [ ctypes.POINTER(ctypes.c_char) ]
DLL.token_string_free.restype = None

#int  DLL_SPEC token_obj_set_string(const struct token_obj *, const char *, size_t size);
DLL.token_obj_set_string.argtypes = [ ctypes.POINTER(Token), ctypes.c_char_p, ctypes.c_size_t ]
DLL.token_obj_set_string.restype = ctypes.c_int
DLL.token_obj_set_string.errcheck = errorOnNegativeFN(lambda r,f,a: TokenManipulationFailure(_obj_strerror(a[0])))

#const struct smbios_struct * DLL_SPEC token_obj_get_smbios_struct(const struct token_obj *);
#DLL.token_obj_get_smbios_struct.argtypes = [ ctypes.POINTER(Token), ]
#DLL.token_obj_get_smbios_struct.restype = ctypes.c_int

#int  DLL_SPEC token_obj_try_password(const struct token_obj *, const char *pass_ascii, const char *pass_scancode);
DLL.token_obj_try_password.argtypes = [ ctypes.POINTER(Token), ctypes.c_char_p, ctypes.c_char_p ]
DLL.token_obj_try_password.restype = ctypes.c_int

#const void * DLL_SPEC token_obj_get_ptr(const struct token_obj *t);
DLL.token_obj_get_ptr.argtypes = [ ctypes.POINTER(Token), ]
DLL.token_obj_get_ptr.restype = ctypes.POINTER(TokenPtr)
DLL.token_obj_get_ptr.errcheck = errorOnNullPtrFN(lambda r,f,a: TokenManipulationFailure(_obj_strerror(r)))



# the purpose of this file is so we can run from the build tree.
# for installation, this is replaced with a better version.
import os

__all__ = ["__VERSION__", "GETTEXT_PACKAGE",
    "localedir", "pkgdatadir", "pythondir", "pkgconfdir", "libdir",
    "LIBSMBIOS_C_SONAME"
    ]

LIBSMBIOS_C_SONAME="libsmbios_c.so.2"

import libsmbios_c
_modpath = os.path.realpath(libsmbios_c.__path__[0])

# the following vars are all substituted on install
__VERSION__="2.4.3"
GETTEXT_PACKAGE="libsmbios-2.4-x86_64"
libdir="/usr/lib"
pythondir="/usr/lib/python3.14/site-packages"
localedir="/usr/share/locale"
pkgdatadir="/usr/share/smbios-utils"
pkgconfdir="/etc/libsmbios"

# end vars




#!/usr/bin/env python3
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:textwidth=0:
# License: GPL2 or later see COPYING
# Written by Michael Brown
# Copyright (C) 2007 Michael E Brown <mebrown@michaels-house.net>

import logging
import os
import string
import sys
import types

import warnings
warnings.filterwarnings('ignore', category=FutureWarning)

def makePrintable(s):
    printable = 1
    for ch in s:
        if ch not in string.printable:
            printable=0

    if printable:
        return s

    else:
        retstr=""
        i = 0
        for ch in s:
            i = i+1
            retstr = retstr + "0x%02x" % ord(ch)
        return retstr

# function decorator that strips trailing whitespace from returned string
def strip_trailing_whitespace():
    def decorator(func):
        def dostrip(*args, **kw):
            result = func(*args, **kw)
            if type(result) is bytes:
                result = result.decode('utf-8')
            while len(result) and result[-1] == " ": result=result[:-1]
            return result
        return dostrip
    return decorator

# defaults to module verbose log
# does a late binding on log. Forwards all attributes to logger.
# works around problem where reconfiguring the logging module means loggers
# configured before reconfig dont output.
class getLog(object):
    def __init__(self, name=None, prefix="", *args, **kargs):
        if name is None:
            frame = sys._getframe(1)
            name = frame.f_globals["__name__"]

        self.name = prefix + name

    def __getattr__(self, name):
        logger = logging.getLogger(self.name)
        return getattr(logger, name)

# emulates logic in logging module to ensure we only log
# messages that logger is enabled to produce.
def doLog(logger, level, *args, **kargs):
    if logger.manager.disable >= level:
        return
    if logger.isEnabledFor(level):
        try:
            logger.handle(logger.makeRecord(logger.name, level, *args, **kargs))
        except TypeError:
            del(kargs["func"])
            logger.handle(logger.makeRecord(logger.name, level, *args, **kargs))

def traceLog(log = None):
    def decorator(func):
        def trace(*args, **kw):
            # default to logger that was passed by module, but
            # can override by passing logger=foo as function parameter.
            # make sure this doesnt conflict with one of the parameters
            # you are expecting

            filename = os.path.normcase(func.__code__.co_filename)
            func_name = func.__code__.co_name
            lineno = func.__code__.co_firstlineno

            l2 = kw.get('logger', log)
            if l2 is None:
                l2 = logging.getLogger("trace.%s" % func.__module__)
            if isinstance(l2, str):
                l2 = logging.getLogger(l2)

            message = "ENTER %s(" % func_name
            for arg in args:
                message = message + repr(arg) + ", "
            for k,v in list(kw.items()):
                message = message + "%s=%s" % (k,repr(v))
            message = message + ")"

            frame = sys._getframe(1)
            doLog(l2, logging.INFO, os.path.normcase(frame.f_code.co_filename), frame.f_lineno, message, args=[], exc_info=None, func=frame.f_code.co_name)
            try:
                result = "Bad exception raised: Exception was not a derived class of 'Exception'"
                try:
                    result = func(*args, **kw)
                except (KeyboardInterrupt, Exception) as e:
                    result = "EXCEPTION RAISED"
                    doLog(l2, logging.INFO, filename, lineno, "EXCEPTION: %s\n" % e, args=[], exc_info=sys.exc_info(), func=func_name)
                    raise
            finally:
                doLog(l2, logging.INFO, filename, lineno, "LEAVE %s --> %s\n" % (func_name, repr(result)), args=[], exc_info=None, func=func_name)

            return result
        return trace
    return decorator

# helper function so we can use back-compat format but not be ugly
def decorateAllFunctions(module, logger=None):
    methods = [ method for method in dir(module)
            if isinstance(getattr(module, method), types.FunctionType)
            ]
    for i in methods:
        setattr(module, i, traceLog(logger)(getattr(module,i)))

# unit tests...
if __name__ == "__main__":
    logging.basicConfig(level=logging.WARNING,
                    format='%(name)s %(levelname)s %(filename)s, %(funcName)s, Line: %(lineno)d:  %(message)s',)
    log = getLog("foobar.bubble")
    root = getLog(name="")
    log.setLevel(logging.WARNING)
    root.setLevel(logging.DEBUG)

    log.debug(" --> debug")
    log.error(" --> error")

    @traceLog(log)
    def testFunc(arg1, arg2="default", *args, **kargs):
        return 42

    testFunc("hello", "world", logger=root)
    testFunc("happy", "joy", name="skippy")
    testFunc("hi")

    @traceLog(root)
    def testFunc22():
        return testFunc("archie", "bunker")

    testFunc22()

    @traceLog(root)
    def testGen():
        yield 1
        yield 2

    for i in testGen():
        log.debug("got: %s" % i)

    @traceLog()
    def anotherFunc(*args):
        return testFunc(*args)

    anotherFunc("pretty")

    getLog()
